package id.go.kemenag.madrasah.pmrms.auth.service

import com.auth0.jwt.JWT
import com.auth0.jwt.algorithms.Algorithm
import com.fasterxml.jackson.databind.DeserializationFeature
import com.fasterxml.jackson.databind.ObjectMapper
import id.go.kemenag.madrasah.pmrms.auth.constant.EXPIRATION_TIME
import id.go.kemenag.madrasah.pmrms.auth.constant.SECRET
import id.go.kemenag.madrasah.pmrms.auth.constant.TOKEN_PREFIX
import id.go.kemenag.madrasah.pmrms.auth.constant.VALIDATOR_MSG_NOT_FOUND
import id.go.kemenag.madrasah.pmrms.auth.exception.BadRequestException
import id.go.kemenag.madrasah.pmrms.auth.helpers.getUserLogin
import id.go.kemenag.madrasah.pmrms.auth.helpers.responseNotFound
import id.go.kemenag.madrasah.pmrms.auth.helpers.responseSuccess
import id.go.kemenag.madrasah.pmrms.auth.model.request.auth.LoginRequest
import id.go.kemenag.madrasah.pmrms.auth.model.response.ReturnData
import id.go.kemenag.madrasah.pmrms.auth.model.response.jwt.JwtAuthenticationResponse
import id.go.kemenag.madrasah.pmrms.auth.pojo.Users
import id.go.kemenag.madrasah.pmrms.auth.repository.UserRepository
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder
import org.springframework.stereotype.Service
import java.util.*

@Service
class AuthService {

    @Autowired
    private lateinit var repoUser: UserRepository

    fun login(request: LoginRequest): ResponseEntity<ReturnData> {
        try {
            val data = repoUser.findByEmailAndActive(request.email)
            if (data.isPresent) {
                if (BCryptPasswordEncoder().matches(request.password, data.get().password)) {
                    val objectMapper = ObjectMapper()
                    objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)

                    val user: Users =
                        objectMapper.readValue(objectMapper.writeValueAsString(data.get()), Users::class.java)
                    return responseSuccess(data = writeToken(user, "user-admin"))
                }
            }
            throw BadRequestException(message = "Pengguna tidak diketahui")
        } catch (e: Exception) {
            throw e
        }
    }

    private fun writeToken(user: Any, audience: String): JwtAuthenticationResponse {
        val objectMapper = ObjectMapper()
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)

        val users: Users = objectMapper.readValue(objectMapper.writeValueAsString(user), Users::class.java)

        val expire = Date(System.currentTimeMillis() + EXPIRATION_TIME)
        val token =
            JWT.create().withSubject(objectMapper.writeValueAsString(user)).withExpiresAt(expire).withAudience(audience)
                .sign(Algorithm.HMAC512(SECRET.toByteArray()))

        val jwtres = JwtAuthenticationResponse(
            token, TOKEN_PREFIX.trim(), expire.time, users
        )

        return jwtres
    }

    fun detail(): ResponseEntity<ReturnData> {
        return try {
            val data = repoUser.findByIdAndActive(getUserLogin()?.id)
            if (data.isPresent) {
                responseSuccess(data = data)
            }
            responseNotFound("User $VALIDATOR_MSG_NOT_FOUND\"")
        } catch (e: Exception) {
            throw e
        }
    }
}
